@@ -1,5 +1,5 @@
 /*
- *   Copyright (C) 2007-2016 Tristan Heaven <tristan@tristanheaven.net>
+ *   Copyright (C) 2007-2020 Tristan Heaven <tristan@tristanheaven.net>
  *
  *   This file is part of GtkHash.
  *
@@ -45,6 +45,9 @@ struct hash_lib_linux_s {
 static const char *gtkhash_hash_lib_linux_get_name(const enum hash_func_e id)
 {
 	switch (id) {
+		case HASH_FUNC_BLAKE2B:   return "blake2b-512";
+		case HASH_FUNC_BLAKE2S:   return "blake2s-256";
+		case HASH_FUNC_CRC32C:    return "crc32c";
 		case HASH_FUNC_MD4:       return "md4";
 		case HASH_FUNC_MD5:       return "md5";
 		case HASH_FUNC_RIPEMD128: return "rmd128";
@@ -75,8 +78,10 @@ bool gtkhash_hash_lib_linux_is_supported(const enum hash_func_e id)
 	if (!(data.name = gtkhash_hash_lib_linux_get_name(id)))
 		return false;
 
-	if ((data.sockfd = socket(AF_ALG, SOCK_SEQPACKET, 0)) == -1)
+	if ((data.sockfd = socket(AF_ALG, SOCK_SEQPACKET, 0)) == -1) {
+		g_message("Kernel AF_ALG: %s", g_strerror(errno));
 		return false;
+	}
 
 	struct sockaddr_alg addr = {
 		.salg_family = AF_ALG,
@@ -86,11 +91,38 @@ bool gtkhash_hash_lib_linux_is_supported(const enum hash_func_e id)
 	strcpy((char *)addr.salg_name, data.name);
 
 	if (bind(data.sockfd, (struct sockaddr *)&addr, sizeof(addr)) == -1) {
-		g_message("kernel AF_ALG '%s' unavailable", data.name);
+		g_message("Kernel AF_ALG '%s': %s", data.name, g_strerror(errno));
 		close(data.sockfd);
 		return false;
 	}
 
+	if ((data.connfd = accept(data.sockfd, NULL, NULL)) == -1) {
+		g_message("Kernel AF_ALG '%s': %s", data.name, g_strerror(errno));
+		close(data.sockfd);
+		return false;
+	}
+
+	// Update
+	ssize_t bytes = 0;
+	if ((bytes = send(data.connfd, "1234567", 8, MSG_MORE)) != 8) {
+		if (bytes < 0)
+			g_message("Kernel AF_ALG '%s': %s", data.name, g_strerror(errno));
+		close(data.connfd);
+		close(data.sockfd);
+		return false;
+	}
+
+	// Finish
+	uint8_t digest[4];
+	if ((bytes = read(data.connfd, &digest, 4)) != 4) {
+		if (bytes < 0)
+			g_message("Kernel AF_ALG '%s': %s", data.name, g_strerror(errno));
+		close(data.connfd);
+		close(data.sockfd);
+		return false;
+	}
+
+	close(data.connfd);
 	close(data.sockfd);
 	return true;
 }
@@ -141,5 +173,9 @@ uint8_t *gtkhash_hash_lib_linux_finish(struct hash_func_s *func, size_t *size)
 	close(LIB_DATA->sockfd);
 	g_free(LIB_DATA);
 
+	// Kernel CRC32C is little-endian
+	if (func->id == HASH_FUNC_CRC32C)
+		*(uint32_t *)digest = GUINT32_SWAP_LE_BE(*(uint32_t *)digest);
+
 	return digest;
 }
