@@ -1,5 +1,5 @@
 /*
- *   Copyright (C) 2007-2018 Tristan Heaven <tristan@tristanheaven.net>
+ *   Copyright (C) 2007-2020 Tristan Heaven <tristan@tristanheaven.net>
  *
  *   This file is part of GtkHash.
  *
@@ -21,14 +21,13 @@
 	#include "config.h"
 #endif
 
-#include <stdlib.h>
-#include <stdbool.h>
-#include <gio/gio.h>
-#include <gtk/gtk.h>
-
 #if IN_NAUTILUS_EXTENSION
-	#include <libnautilus-extension/nautilus-property-page.h>
-	#include <libnautilus-extension/nautilus-property-page-provider.h>
+	#if HAVE_NAUTILUS_EXTENSION_H
+		#include <nautilus-extension.h>
+	#else
+		#include <libnautilus-extension/nautilus-property-page.h>
+		#include <libnautilus-extension/nautilus-property-page-provider.h>
+	#endif
 #elif IN_CAJA_EXTENSION
 	#include <libcaja-extension/caja-property-page.h>
 	#include <libcaja-extension/caja-property-page-provider.h>
@@ -40,9 +39,15 @@
 	#include <libpeony-extension/peony-property-page.h>
 	#include <libpeony-extension/peony-property-page-provider.h>
 #elif IN_THUNAR_EXTENSION
+	#undef GTK_DISABLE_DEPRECATED // thunarx-3 doesn't build with this
 	#include <thunarx/thunarx.h>
 #endif
 
+#include <stdlib.h>
+#include <stdbool.h>
+#include <gio/gio.h>
+#include <gtk/gtk.h>
+
 #include "properties.h"
 #include "properties-list.h"
 #include "properties-hash.h"
@@ -151,11 +156,7 @@ static void gtkhash_properties_on_treeview_popup_menu(struct page_s *page)
 static bool gtkhash_properties_on_treeview_button_press_event(
 	struct page_s *page, GdkEventButton *event)
 {
-#if (GTK_MAJOR_VERSION > 2)
 	if (gdk_event_triggers_context_menu((GdkEvent *)event))
-#else
-	if ((event->type == GDK_BUTTON_PRESS) && (event->button == 3))
-#endif
 #if GTK_CHECK_VERSION(3,22,0)
 		gtk_menu_popup_at_pointer(page->menu, (GdkEvent *)event);
 #else
@@ -166,6 +167,21 @@ static bool gtkhash_properties_on_treeview_button_press_event(
 	return false;
 }
 
+static void gtkhash_properties_on_treeview_row_activated(
+	struct page_s *page, GtkTreePath *path, GtkTreeViewColumn *column,
+	G_GNUC_UNUSED GtkTreeView *treeview)
+{
+	// Ignore checkbutton column
+	if (!*gtk_tree_view_column_get_title(column))
+		return;
+
+	if (!gtk_tree_selection_path_is_selected(page->treeselection, path))
+		return;
+
+	if (gtkhash_properties_list_hash_selected(page))
+		gtkhash_properties_busy(page);
+}
+
 static void gtkhash_properties_on_menu_map_event(struct page_s *page)
 {
 	bool sensitive = false;
@@ -199,6 +215,20 @@ static void gtkhash_properties_on_entry_check_changed(struct page_s *page)
 	gtkhash_properties_list_check_digests(page);
 }
 
+static void gtkhash_properties_on_entry_check_icon_press(GtkEntry *entry,
+	GtkEntryIconPosition pos, GdkEventButton *event)
+{
+	if (pos != GTK_ENTRY_ICON_PRIMARY)
+		return;
+	if (event->type != GDK_BUTTON_PRESS)
+		return;
+	if (event->button != 1)
+		return;
+
+	gtk_entry_set_text(entry, "");
+	gtk_editable_paste_clipboard(GTK_EDITABLE(entry));
+}
+
 static void gtkhash_properties_on_entry_hmac_changed(struct page_s *page)
 {
 	gtkhash_properties_list_clear_digests(page);
@@ -223,8 +253,8 @@ static void gtkhash_properties_on_entry_hmac_populate_popup(GtkEntry *entry,
 	gtk_widget_show(item);
 	gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
 
-	// Add checkbutton
 	item = gtk_check_menu_item_new_with_mnemonic(_("_Show HMAC Key"));
+	// Add checkbutton
 	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(item),
 		gtk_entry_get_visibility(entry));
 	gtk_widget_show(item);
@@ -250,9 +280,9 @@ static void gtkhash_properties_on_button_hash_clicked(struct page_s *page)
 			page->entry_hmac);
 		GtkEntryBuffer *buffer = gtk_entry_get_buffer(page->entry_hmac);
 		const size_t key_size = gtk_entry_buffer_get_bytes(buffer);
-		gtkhash_properties_hash_start(page, hmac_key, key_size);
+		gtkhash_properties_hash_start(page, NULL, hmac_key, key_size);
 	} else
-		gtkhash_properties_hash_start(page, NULL, 0);
+		gtkhash_properties_hash_start(page, NULL, NULL, 0);
 }
 
 static void gtkhash_properties_on_button_stop_clicked(struct page_s *page)
@@ -320,10 +350,6 @@ static void gtkhash_properties_init_objects(struct page_s *page,
 		"button_hash"));
 	page->button_stop = GTK_BUTTON(gtkhash_properties_get_object(builder,
 		"button_stop"));
-#if (GTK_MAJOR_VERSION > 2)
-	gtk_button_set_always_show_image(page->button_hash, true);
-	gtk_button_set_always_show_image(page->button_stop, true);
-#endif
 }
 
 static void gtkhash_properties_connect_signals(struct page_s *page)
@@ -339,6 +365,8 @@ static void gtkhash_properties_connect_signals(struct page_s *page)
 		G_CALLBACK(gtkhash_properties_on_treeview_popup_menu), page);
 	g_signal_connect_swapped(page->treeview, "button-press-event",
 		G_CALLBACK(gtkhash_properties_on_treeview_button_press_event), page);
+	g_signal_connect_swapped(page->treeview, "row-activated",
+		G_CALLBACK(gtkhash_properties_on_treeview_row_activated), page);
 
 	// Popup menu
 	g_signal_connect_swapped(page->menu, "map-event",
@@ -351,6 +379,8 @@ static void gtkhash_properties_connect_signals(struct page_s *page)
 	// Check
 	g_signal_connect_swapped(page->entry_check, "changed",
 		G_CALLBACK(gtkhash_properties_on_entry_check_changed), page);
+	g_signal_connect(page->entry_check, "icon-press",
+		G_CALLBACK(gtkhash_properties_on_entry_check_icon_press), NULL);
 
 	// HMAC
 	g_signal_connect_swapped(page->togglebutton_hmac, "toggled",
@@ -367,43 +397,9 @@ static void gtkhash_properties_connect_signals(struct page_s *page)
 		G_CALLBACK(gtkhash_properties_on_button_stop_clicked), page);
 }
 
-static GtkBuilder *gtkhash_properties_init_builder(void)
-{
-#if (GTK_MAJOR_VERSION > 2)
-	return gtk_builder_new_from_resource(PROPERTIES_XML_RESOURCE);
-#else
-	GError *error = NULL;
-	GBytes *bytes = g_resources_lookup_data(PROPERTIES_XML_RESOURCE,
-		G_RESOURCE_LOOKUP_FLAGS_NONE, &error);
-
-	if (G_UNLIKELY(error)) {
-		g_warning("%s", error->message);
-		g_error_free(error);
-		return NULL;
-	}
-
-	gsize xml_len = 0;
-	char *xml = g_bytes_unref_to_data(bytes, &xml_len);
-	GtkBuilder *builder = gtk_builder_new();
-
-	gtk_builder_add_from_string(builder, xml, xml_len, &error);
-	g_free(xml);
-
-	if (G_UNLIKELY(error)) {
-		g_warning("%s", error->message);
-		g_error_free(error);
-		g_object_unref(builder);
-
-		return NULL;
-	}
-
-	return builder;
-#endif
-}
-
 static struct page_s *gtkhash_properties_new_page(char *uri)
 {
-	GtkBuilder *builder = gtkhash_properties_init_builder();
+	GtkBuilder *builder = gtk_builder_new_from_resource(PROPERTIES_XML_RESOURCE);
 	if (!builder)
 		return NULL;
 
@@ -480,18 +476,18 @@ static GList *gtkhash_properties_get_pages(
 
 #if IN_NAUTILUS_EXTENSION
 	NautilusPropertyPage *ppage = nautilus_property_page_new(
-		"GtkHash::properties", gtk_label_new(_("Digests")), page->box);
+		"GtkHash::properties", gtk_label_new(_("Checksums")), page->box);
 #elif IN_CAJA_EXTENSION
 	CajaPropertyPage *ppage = caja_property_page_new(
-		"GtkHash::properties", gtk_label_new(_("Digests")), page->box);
+		"GtkHash::properties", gtk_label_new(_("Checksums")), page->box);
 #elif IN_NEMO_EXTENSION
 	NemoPropertyPage *ppage = nemo_property_page_new(
-		"GtkHash::properties", gtk_label_new(_("Digests")), page->box);
+		"GtkHash::properties", gtk_label_new(_("Checksums")), page->box);
 #elif IN_PEONY_EXTENSION
 	PeonyPropertyPage *ppage = peony_property_page_new(
-		"GtkHash::properties", gtk_label_new(_("Digests")), page->box);
+		"GtkHash::properties", gtk_label_new(_("Checksums")), page->box);
 #elif IN_THUNAR_EXTENSION
-	GtkWidget *ppage = thunarx_property_page_new(_("Digests"));
+	GtkWidget *ppage = thunarx_property_page_new(_("Checksums"));
 	gtk_container_add(GTK_CONTAINER(ppage), page->box);
 #endif
 
